rem
rem $Header: estat.sql,v 6002700.1 89/05/22 17:06:46 rcs Exp $ estat.sql Copyr (c) 1989 Oracle
rem
rem RDBMS660027,DISK$DEV1:[V6SOURCE.UTL.60027]
Rem Copyright (c) 1988 by Oracle Corporation
Rem NAME
Rem    ESTAT.SQL - collect Ending STATistics
Rem  FUNCTION
Rem    This script will generate a report (in "report.txt") which will contain
Rem    usefull information for performance monitoring.  In particular
Rem    information from v$sysstat, v$latch, and v$rollstat.
Rem  NOTES
Rem    Don't worry about errors during "drop table"s, they are normal.
Rem  MODIFIED
Rem   Loaiza     04/04/89 - fix run dates to do minutes instead of months
Rem   Loaiza     03/31/89 - add kqrst usage column
Rem   Jloaiza    03/16/89 - improve names and formats
Rem   Jloaiza    03/09/89 - make kqrst columns intelligible
Rem   Jloaiza    02/23/89 - changed table names, added dates
Rem   Martin     02/22/89 - Creation
set echo on;
connect internal;

drop table stats$stats;
drop table stats$latches;
drop table stats$roll;
drop table stats$files;
drop table stats$kqrst;
drop table stats$end_stats;
drop table stats$end_latch;
drop table stats$end_roll;
drop table stats$end_file;
drop table stats$end_kqrst;
create table stats$end_stats as select * from v$sysstat;
create table stats$end_latch as select * from v$latch;
create table stats$end_roll as select rownum undo#,rssize,gets,waits,writes from v$rollstat;
create table stats$end_file as select * from stats$file_view;
create table stats$end_kqrst as select * from x$kqrst;

create table stats$stats as
select  e.value-b.value change , n.name
	from v$statname n ,  stats$begin_stats b , stats$end_stats e
	where n.statistic# = b.statistic# and n.statistic# = e.statistic#;

create table stats$latches as
select 
	e.waits-b.waits waits, 
	e.immediates-b.immediates immediates,
	e.timeouts-b.timeouts timeouts,
	e.nowaits-b.nowaits nowaits,
	e.successes-b.successes successes ,
	n.name
	from v$latchname n ,  stats$begin_latch b , stats$end_latch e
	where n.latch# = b.latch# and n.latch# = e.latch#;

create table stats$roll as
select  e.gets-b.gets trans_tbl_gets, 
	e.waits-b.waits trans_tbl_waits, 
	e.writes-b.writes undo_bytes_written,
	e.rssize segment_size_bytes
	from stats$begin_roll b , stats$end_roll e
        where e.undo# = b.undo#;

create table stats$files as
select b.ts table_space,
       b.name file_name,
       e.pyr-b.pyr phys_reads,
       e.pbr-b.pbr phys_blks_rd,
       e.prt-b.prt phys_rd_time,
       e.pyw-b.pyw phys_writes,
       e.pbw-b.pbw phys_blks_wr,
       e.pwt-b.pwt phys_wrt_tim
  from stats$begin_file b, stats$end_file e
 where b.name=e.name;

create table stats$kqrst as
select 
       b.kqrsttxt name,
       e.kqrstgrq-b.kqrstgrq get_reqs,
       e.kqrstgmi-b.kqrstgmi get_miss,
       e.kqrstsrq-b.kqrstsrq scan_reqs,
       e.kqrstsmi-b.kqrstsmi scan_miss,
       e.kqrstmrq-b.kqrstmrq mod_reqs,
       e.kqrstusg cur_usage
from stats$begin_kqrst b, stats$end_kqrst e
where b.indx=e.indx;

insert into stats$dates 
   select to_char(sysdate, 'dd-mon-yy hh:mi:ss') from stats$dual;

spool report.txt;
set charwidth 30;
set numwidth 11;
rem The total is the total value of the statistic between the time
rem bstat was run and the time estat was run.
select n1.name "Statistic", 
       n1.change "Total", 
       trunc(n1.change/n2.change,2) "Per Trans" 
   from stats$stats n1, stats$stats n2
   where n2.name='user commits'
   order by n1.name;
set charwidth 48;
set numwidth 12;
rem I/O should be spread evenly accross drives. A big difference between
rem phys_reads and phys_blks_rd implies table scans are going on.
select * from stats$files;
set charwidth 26;
set numwidth 9;
rem Timeouts should be low.  Successes should be very close to nowaits.
select name, waits, immediates, timeouts, nowaits, successes from stats$latches order by name;
set numwidth 19;
rem Waits_for_trans_tbl high implies you should add rollback segments.
select * from stats$roll;
set charwidth 20;
set numwidth 8;
rem get_miss and scan_miss should be very low compared to the requests.
rem cur_usage is the number of entries in the cache that are being used.
select * from stats$kqrst;
rem The init.ora parameters currently in effect:
show parameters;
rem The times that bstat and estat were run.
select * from stats$dates;
spool off;

drop table stats$dual;
drop table stats$dates;
drop table stats$begin_stats;
drop table stats$begin_latch;
drop table stats$begin_roll;
drop table stats$begin_file;
drop table stats$begin_kqrst;
drop view stats$file_view;
drop table stats$stats;
drop table stats$latches;
drop table stats$roll;
drop table stats$files;
drop table stats$kqrst;
drop table stats$end_stats;
drop table stats$end_latch;
drop table stats$end_roll;
drop table stats$end_file;
drop table stats$end_kqrst;
disconnect;

