rem 
rem $Header: expvew5.sql,v 6002300.1 89/06/16 01:22:15 cyang Exp $ expvew5.sql Copyr (c) 1987 Oracle
rem 
rem RDBMS660023,DISK$DEV1:[V6SOURCE.OLB.60023]
remark  Copyright (c) Oracle Corporation 1988
remark
remark  NAME
remark    EXPVEW5.SQL
remark  FUNCTION
remark    Creates V5.1-style export views so that EXP V5.1 can be 
remark    used over SQL*Net to export against V6.0 for import into V5.1.
remark  NOTES
remark       These views are installed in SYS account.
remark       Many of the views created by this file have the same names as
remark    views in the file CATALOG5.SQL, which contains V5-compatible
remark    data dictionary views.  If those views were created, these views
remark    will replace them.
remark **    Because of a 'bug' in v6, exporting more than 1 users in
remark    user mode doesn't work (bug 13386).
remark **    Due to changes in the password encryption algorithm, all 
remark    encrypted passwords are returned as the literal 
remark    'REGRANT_AFTER_IMPORT' except for the user PUBLIC, which has a
remark    NULL password.  A NULL password prevents EXPORT from generating 
remark    GRANT statements for the user or exporting the user's objects. 
remark    The user SYS is explicitly excluded.  SYSTEM will have an invalid
remark    password changed to garbage, meaning that SYS will be the only 
remark    valid logon in the database.  Someone will have to log on as
remark    SYS and reGRANT CONNECT to all the other users in order to 
remark    restore their logons.
remark **   Since EXP uses connect-by to generate to enumerate the
remark    table authorizations, we cannot create TABAUTH as a view
remark    as it involves joins. Instead it is created as a table to
remark    so the connect-by works. Therefore, one must RECREATE
remark    SYS.TABAUTH before a full database export.
remark **    Storage parameters for individual indexes will be lost, since 
remark    this information is not maintained in V5.  The default index
remark    parameters for the tablespace are used for the INDEXPAGES of the
remark    for the table.
remark       The following column datatype conversions were required:
remark    VARCHAR -> CHAR
remark    ROWID (V6, type# = 69) -> ROWID (V5, type# = 11).
remark **    Column defaults are ignored.
remark **    All indexes are considered compressed, as all indexes are 
remark    compressed in V6.  The IND$.COMPRESS$ column in the data
remark    dictionary does not seem to be maintained, so the proper value
remark    is hardcoded here.
remark       V6 does not impose the V5 restriction on clustered tables
remark    whereby such tables had to have at least one column in the 
remark    cluster key defined as NOT NULL.  No attempt is made here to
remark    address this incompatibility, in part because an export file
remark    created using the V6-to-V5 method may be imported back into V6,
remark    where such tables are legal.  Attempts to import into V5 tables
remark    that do not comply with this restriction will fail.  The
remark    table, however, can be pre-created unclustered, or can be
remark    exported in table mode, which will uncluster it.
remark       Since in V5 the minimum extent size was 3 blocks, extent sizes
remark    of 1 and 2 blocks are converted to three blocks where feasible.
remark    The one remaining possible cause of failure is as follows:
remark    if a table, cluster, or the sum of all an object's indexes have
remark    very little storage allocated to them, i.e., 1 or 2 blocks, then
remark    if the "compress extents" export option is used, EXP will 
remark    generate an ALTER SPACE DEFINITION statement that asks for an
remark    initial extent of less than 3 blocks.  V5 IMP will report an
remark    error on the statement, but will still be able to import the
remark    object (using the previous object's space definition).  It is
remark    possible to write the affected views, EXTENTS and V4EXPEXTENTS,
remark    in such a way as to avoid this circumstance, but this would
remark    make them even slower (they already are 5-way UNIONs).  The
remark    circumstance should be rare, so the current decision seems to
remark    be the right one.
remark       PCTFREE of zero is valid in V6; it is not in V5.  PCTFREE of
remark    zero is converted to 1.
remark       COLUMNS and V4EXPCOL return SEGCOL# (column number as stored
remark    in the segment) rather than COL# (column number as created)
remark    for the column id because that is the way it was stored in
remark    V5.  This fact required a special fix to EXP to get it
remark    to export clustered tables correctly. (isn't this fixed? chin)
remark 
remark       See comments before view definitions for other caveats.
remark    Many views come in pairs, e.g., INDEXES and V4EXPINDEX.  Comments
remark    for one view of a pair generally apply for the other.
remark    
remark  MODIFICATION
remark   Bulchan        10/06/88 - Remove connect
remark    Hong          05/23/88 - make it work for TPS v6.0.19,
remark                             rename to expvew5.sql.
remark    Grayson       05/19/88 - Updated comments, changed exported 
remark                             encrypted passwords
remark    Grayson       05/18/88 - Renamed file
remark                             V6TOV5EXPVIEWS.SQL -> EXP5VIEW.SQL
remark    Grayson       03/26/88 - Creation
remark
remark  Because V5 uses a user number for non-remote synonyms,
remark  such synonyms which reference users that do not exist will not
remark  be exported.
remark    In V5, the pid/rba/tbl was used to identify tables, views, and
remark    and clusters.  V6 replaced this compound key with a single obj#.
remark    The following adaptations are used here:
remark    Tables:    pid/rba/tbl -> ts#/obj#/0
remark    Clusters:  pid/rba/tbl -> ts#/obj#/0
remark    Views:     pid/rba/tbl -> 0/obj#/0
remark    Synonyms:  pid/rba/tbl -> 0/obj#/0
remark
remark    The SIZE parameter to the CREATE CLUSTER statement has a somewhat
remark    different meaning in V6.  They are sufficiently similar, however,
remark    so that they may be used interchangably.  If the SIZE stored in
remark    the CLU$ table is NULL, the blocksize for the tablespace is used.
remark
remark    The clusters belonging to SYS are excluded, because otherwise
remark    EXP will export them.
remark
set echo on;
drop view tables
/
create view tables
    (tab$pid, tab$rba, tab$tbl,
     tab$owner, tab$name, tab$type, tab$logblk,
     tab$cluster, tab$reqblk, tab$indcompr,
     tab$time,
     tab$sowner, tab$sname,
     tab$audit, tab$comment, tab$snode, tab$sownername)
as
select t.ts#, o.obj#, 0,
       o.owner#, o.name, 'TABLE', to_number(null),
       to_number(null), to_number(null), to_char(null),
       o.ctime,
       to_number(null), to_char(null),
       substr(t.audit$,1,22), c.comment$, to_char(null), to_char(null)
from sys.obj$ o, sys.tab$ t, sys.com$ c
where o.obj# = t.obj# and o.obj# = c.obj#(+) and c.col#(+) is null
  and o.type = 2
union
select 0, o.obj#, 0,
       o.owner#, o.name, 'VIEW', to_number(null),
       to_number(null), to_number(null), to_char(null),
       o.ctime,
       to_number(null), to_char(null),
       substr(v.audit$, 1, 22), to_char(null), to_char(null), to_char(null)
from sys.obj$ o, sys.view$ v
where o.obj# = v.obj#
  and o.type = 4
union
select c.ts#, c.obj#, 0,
       o.owner#, o.name, 'CLUSTER', nvl(c.size$, ts.blocksize),
       to_number(null), nvl(c.size$, ts.blocksize), 'COMPRESS',
/*     decode(i.compress$, 0, 'NOCOMPRESS', 1, 'COMPRESS'), */
       o.ctime,
       to_number(null), to_char(null),
       to_char(null), to_char(null), to_char(null), to_char(null)
from sys.obj$ o, sys.clu$ c, sys.ts$ ts /*, sys.ind$ i */
where o.obj# = c.obj#
  and ts.ts# = c.ts#
/*and c.obj# = i.bo# (+) */
  and o.type = 3
  and o.owner# != 0
union
select 0, o.obj#, 0,
       o.owner#, o.name, 'SYNONYM', to_number(null),
       to_number(null), to_number(null), to_char(null),
       o.ctime,
       u.user#, s.name,
       to_char(null), to_char(null), s.owner, s.node
from sys.obj$ o, sys.syn$ s, sys.user$ u
where o.obj# = s.obj#
  and o.type = 5
  and s.owner = u.name(+)
/
remark
remark Column view
remark
drop view columns
/
create view columns
    (col$pid, col$rba, col$tbl,
     col$name,
     col$datatype,
     col$length, col$scale,
     col$null_type,
     col$id, col$comment, col$default)
as
select t.ts#, c.obj#, 0,
       c.name,
       decode(c.type#, 1, 'CHAR', 2, 'NUMBER', 8, 'LONG', 9, 'CHAR',
                       11, 'ROWID', 12, 'DATE', 23, 'RAW', 24, 'LONG RAW',
                       69, 'ROWID'),
       decode(c.type#, 2, nvl(c.precision, 0), c.length), c.scale,
       decode(c.null$, 0, 'NULL', 'NOT NULL'),
       c.segcol#, cc.comment$, to_char(null)
from sys.col$ c, sys.com$ cc, sys.tab$ t
where  c.obj# = cc.obj#(+)
  and  c.col# = cc.col#(+)
  and  t.obj# = c.obj#
union
select cl.ts#, c.obj#, 0,
       c.name,
       decode(c.type#, 1, 'CHAR', 2, 'NUMBER', 8, 'LONG', 9, 'CHAR',
                       11, 'ROWID', 12, 'DATE', 23, 'RAW', 24, 'LONG RAW',
                       69, 'ROWID'),
       decode(c.type#, 2, nvl(c.precision, 0), c.length), c.scale,
       decode(c.null$, 0, 'NULL', 'NOT NULL'),
       c.segcol#, cc.comment$, to_char(null)
from sys.col$ c, sys.com$ cc, sys.clu$ cl
where  c.obj# = cc.obj#(+)
  and  c.col# = cc.col#(+)
  and  cl.obj# = c.obj#
union
select 0, c.obj#, 0,
       c.name,
       decode(c.type#, 1, 'CHAR', 2, 'NUMBER', 8, 'LONG', 9, 'CHAR',
                       11, 'ROWID', 12, 'DATE', 23, 'RAW', 24, 'LONG RAW',
                       69, 'ROWID'),
       decode(c.type#, 2, nvl(c.precision, 0), c.length), c.scale,
       decode(c.null$, 0, 'NULL', 'NOT NULL'),
       c.segcol#, cc.comment$, to_char(null)
from sys.col$ c, sys.com$ cc, sys.obj$ o
where  c.obj# = cc.obj#(+)
  and  c.col# = cc.col#(+)
  and  o.obj# = c.obj# 
  and  o.type = 4
/
remark
remark  EXP expects the EXTENTS view to return all the extents associated with
remark  a table for its data and indexes.  Finding the data extents is
remark  easy enough, but since indexes are dissociated from their tables
remark  or clusters in V6, the third queries below associate
remark  a table or cluster, respectively, to the extents allocated for
remark  that object's indexes.  In V5, every table or cluster had at
remark  least one extent in its index segment.  In V6, however, any table
remark  or cluster with no indexes on it will naturally have no index
remark  extents assocated with it.  When EXP "compresses extents" it sums
remark  the lengths of the extents for each type, and if there are no
remark  index extents, the sum for the indexes would naturally be zero.
remark  This zero result is unacceptable because it is illegal in V5
remark  to use an initial extent of zero in a space definition.
remark  Thus, the last queries below create a phony index
remark  extent for each table and cluster with no indexes. 
remark
drop view extents
/
create view extents
    (ext$pid, ext$rba, ext$type,
     ext$startblock, ext$endblock)
as
select t.ts#, t.obj#, 'DATA',
       e.block#, e.block# + e.length - 1
from sys.uet$ e, sys.tab$ t
where t.file# = e.segfile#
  and t.block# = e.segblock#
union
select cl.ts#, cl.obj#, 'DATA',
       e.block#, e.block# + e.length - 1
from sys.uet$ e, sys.clu$ cl
where cl.file# = e.segfile#
  and cl.block# = e.segblock#
union
select i.ts#, i.bo#, 'INDEX',
       e.block#, e.block# + e.length - 1
from sys.uet$ e, sys.ind$ i
where i.file# = e.segfile#
  and i.block# = e.segblock#
union
select t.ts#, t.obj#, 'INDEX',
       1, decode(ts.dflinit, 1, 3, 2, 3, ts.dflinit)
from sys.tab$ t, sys.ts$ ts
where t.ts# = ts.ts#
  and t.obj# not in
        (select i.bo#
         from sys.ind$ i)
union
select c.ts#, c.obj#, 'INDEX',
       1, decode(ts.dflinit, 1, 3, 2, 3, ts.dflinit)
from sys.clu$ c, sys.ts$ ts
where c.ts# = ts.ts#
  and c.obj# not in
        (select i.bo#
         from sys.ind$ i)
/
remark
remark Table Authorizations
remark
drop table tabauth
/
create table tabauth
  (tau$grantor, tau$grantee, tau$creator,
   tau$tname, tau$time, tau$alter, tau$delete,
   tau$index, tau$insert, tau$select, tau$update)
as
select ur.name, ue.name, uo.name, o.name, ta.time,
       decode(ta.alter$,  0, ' ', 1, 'Y', 2, 'Y', 3, 'G', ' '),
       decode(ta.delete$, 0, ' ', 1, 'Y', 2, 'Y', 3, 'G', ' '),
       decode(ta.index$,  0, ' ', 1, 'Y', 2, 'Y', 3, 'G', ' '),
       decode(ta.insert$, 0, ' ', 1, 'Y', 2, 'Y', 3, 'G', ' '),
       decode(ta.select$, 0, ' ', 1, 'Y', 2, 'Y', 3, 'G', ' '),
       decode(ta.update$, 0, ' ', 1, '*', 2, 'Y', 3, 'G', ' ')
from sys.obj$ o, sys.user$ ur, sys.user$ ue, sys.user$ uo,sys.tabauth$ ta
where o.obj# = ta.obj#
  and ur.user# = ta.grantor#
  and ue.user# = ta.grantee#
  and uo.user# = o.owner#
  and o.type in (2, 4)
/
remark
remark Views table
remark
drop view views
/
create view views
    (vew$pid, vew$rba, vew$tbl,
     vew$textlen, vew$text)
as
select 0,o.obj#, 0,
       v.textlength, v.text
from sys.obj$ o, sys.view$ v
where o.obj# = v.obj#
  and o.type = 4
/
remark
remark  Since there are no space definitions in V6, we create an empty
remark  space definitions table.
remark
drop table spaces
/
create table spaces
  (
  spc$name         char(30)       not null,
  spc$dini         number         not null,
  spc$dinc         number         not null,
  spc$dmextnt      number         not null,
  spc$iini         number         not null,
  spc$iinc         number         not null,
  spc$imextnt      number         not null,
  spc$pctfree      number         not null,
  spc$pid          number
  )
  storage (initial 4k next 4k maxextents 1 pctincrease 0)
/
remark
remark   This view suppresses cluster indexes as they are created
remark  automatically in V5.
remark
remark  V6 does not support the syntax for defining ascending and descending 
remark  indexes.  All indexes are returned as 'ASC', because all indexes
remark  were ascending in V5 anyway ('DESC' was a no-op).
remark
remark  Since EXP uses the IDX$CONCATID field only to determine whether an
remark  index is concatenated (= 1) or not (= 0), only those values are
remark  returned here.
remark
drop view indexes
/
create view indexes
    (idx$pid, idx$rba, idx$tbl,
     idx$owner, idx$name, idx$colseq, idx$column,
     idx$unique,
     idx$order,
     idx$compress,
     idx$concatid)
as
select i.ts#, i.bo#, 0,
       o.owner#, o.name, ic.pos#, c.name,
       decode(i.unique$, 0, 'NON UNIQUE', 1, 'UNIQUE'),
       'ASC',
       'COMPRESS',
    /* decode(i.compress$, 0, 'NOCOMPRESS', 1, 'COMPRESS'), */
       decode(i.cols, 1, 0, 1)
from sys.tab$ t, sys.obj$ o, sys.ind$ i, sys.icol$ ic, sys.col$ c
where o.obj# = i.obj#
  and ic.obj# = i.obj#
  and t.obj# = i.bo#
  and c.obj# = i.bo#
  and c.col# = ic.col#
  and o.type = 1
/
remark
remark  This view excludes the placeholder "_NEXT_USER", because otherwise
remark  EXP will try to export it.  "PUBLIC" has a null password to prevent
remark  EXP from generating a GRANT CONNECT statement for it.
remark
drop view userauth
/
create view userauth
    (usr$uid, usr$name,
     usr$password,
     usr$time,
     usr$connect,
     usr$dba,
     usr$resource)
as
select u.user#, u.name,
       decode(u.name, 'PUBLIC', to_char(null), 'REGRANT_ON_IMPORT'),
       u.ctime,
       decode(u.connect$, 0, ' ', 1, 'Y'),
       decode(u.dba$, 0, ' ', 1, 'Y'),
       decode(u.resource$, 0, ' ', 1, 'Y')
from sys.user$ u
where u.name != '_NEXT_USER'
/
remark
remark  Map tablespaces into partitions
remark
drop view partitions
/
create view partitions
  (par$id, par$name)
as
select ts.ts#, ts.name
from sys.ts$ ts
/
remark
remark  Since, in V6, index storage parameters are not associated with the
remark  table, as in V5, but with the tablespace, the default storage
remark  parameters for the tablespace are returned here for index quotas.
remark
drop view segquotas
/
create view segquotas
    (sgq$pid, sgq$rba, sgq$tbl,
     sgq$dini, sgq$dinc, sgq$dmextnt,
     sgq$iini, sgq$iinc, sgq$imextnt,
     sgq$pctfree, sgq$date)
as
select t.ts#, o.obj#, 0,
       decode(s.iniexts, 1, 3, 2, 3, s.iniexts),
       decode(s.extsize, 1, 3, 2, 3, s.extsize),
       s.maxexts,
       decode(ts.dflinit, 1, 3, 2, 3, ts.dflinit),
       decode(ts.dflincr, 1, 3, 2, 3, ts.dflincr),
       ts.dflmaxext,
       decode(t.pctfree$, 0, 1, t.pctfree$), o.ctime
from sys.seg$ s, sys.tab$ t, sys.ts$ ts, sys.obj$ o
where s.file# = t.file#
  and s.block# = t.block#
  and s.ts# = ts.ts#
  and t.obj# = o.obj#
  and s.type = 5
  and o.type = 2
union
select c.ts#, c.obj#, 0,
       decode(s.iniexts, 1, 3, 2, 3, s.iniexts),
       decode(s.extsize, 1, 3, 2, 3, s.extsize),
       s.maxexts,
       decode(ts.dflinit, 1, 3, 2, 3, ts.dflinit),
       decode(ts.dflincr, 1, 3, 2, 3, ts.dflincr),
       ts.dflmaxext,
       decode(c.pctfree$, 0, 1, c.pctfree$), o.ctime
from sys.seg$ s, sys.clu$ c, sys.ts$ ts, sys.obj$ o
where s.file# = c.file#
  and s.block# = c.block#
  and s.ts# = ts.ts#
  and c.obj# = o.obj#
  and s.type = 5
/
remark
remark Views to get at current user's objects
remark
drop view v4expuser
/
create view v4expuser (myid, mypsswd, myprivs, myname) as
  select u.user#, decode(u.name, 'PUBLIC', to_char(null), 'REGRANT_ON_IMPORT'),
  decode(u.connect$, 0, ' ', 1, 'Y', ' ') ||
    decode(u.dba$, 0, ' ', 1, 'Y', ' ') ||
    decode(u.resource$, 0, ' ', 1, 'Y', ' '),
  user
from sys.user$ u
where u.user# = uid
/
grant select on v4expuser to public
/
remark
remark  The default parameters for the tablespace are used for index 
remark  storage, since index storage is not associated with the table as in V5.
remark  The name of the table's tablespace is returned as the partition 
remark  name, but the file# is returned instead
remark  of the ts#, since the file#/block# concept replaces pid/rba.
remark
drop view v4expspace
/
create view v4expspace
  (dini, dinc, dmax, iini, iinc, imax, pctf, pname, pid, rba, tbl) as
select decode(s.iniexts, 1, 3, 2, 3, s.iniexts),
       decode(s.extsize, 1, 3, 2, 3, s.extsize),
       s.maxexts,
       decode(ts.dflinit, 1, 3, 2, 3, ts.dflinit),
       decode(ts.dflincr, 1, 3, 2, 3, ts.dflincr),
       ts.dflmaxext,
       decode(t.pctfree$, 0, 1, t.pctfree$),
       ts.name, t.ts#, t.obj#, 0
from sys.seg$ s, sys.tab$ t, sys.ts$ ts, sys.obj$ o
where s.file# = t.file#
  and s.block# = t.block#
  and s.ts# = ts.ts#
  and t.obj# = o.obj#
  and s.type = 5
  and o.type = 2
  and s.user# = uid
union
select decode(s.iniexts, 1, 3, 2, 3, s.iniexts),
       decode(s.extsize, 1, 3, 2, 3, s.extsize),
       s.maxexts,
       decode(ts.dflinit, 1, 3, 2, 3, ts.dflinit),
       decode(ts.dflincr, 1, 3, 2, 3, ts.dflincr),
       ts.dflmaxext,
       decode(c.pctfree$, 0, 1, c.pctfree$),
       ts.name, c.ts#, c.obj#, 0
from sys.seg$ s, sys.clu$ c, sys.ts$ ts, sys.obj$ o
where s.file# = c.file#
  and s.block# = c.block#
  and s.ts# = ts.ts#
  and c.obj# = o.obj#
  and s.type = 5
  and o.type = 3
  and s.user# = uid
/
grant select on v4expspace to public
/
drop view v4expindex
/
create view v4expindex
  (tname, usrname, usrid, iname, colname, iunique, icompr, cid, cseq) as
select ot.name, user, uid, oi.name, c.name,
       i.unique$,
       1,
    /* i.compress$, */
       decode(i.cols, 1, 0, 1), ic.pos#
from sys.obj$ ot, sys.obj$ oi, sys.ind$ i, sys.icol$ ic, sys.col$ c
where oi.obj# = i.obj#
  and ic.obj# = i.obj#
  and ot.obj# = i.bo#
  and c.obj# = i.bo#
  and c.col# = ic.col#
  and oi.type = 1
  and ot.type = 2
  and oi.owner# = uid
  and ot.owner# = uid
/
grant select on v4expindex to public
/
drop view v4exptab
/
create view v4exptab (name, pid, rba, tbl, aud, cmnt) as
select o.name, t.ts#, t.obj#, 0, substr(t.audit$,1,22), c.comment$
from sys.obj$ o, sys.tab$ t, sys.com$ c
where o.obj# = t.obj#
  and o.type = 2
  and  o.obj# = c.obj#(+) 
  and c.col#(+) is null
  and o.owner# = uid
/
grant select on v4exptab to public
/
remark
remark  Clusters don't have table numbers (tbl) in V6.  It always was zero
remark  in V5.
remark
drop view v4expclus
/
create view v4expclus (name, pid, rba, tbl, compr, reqblk, owner) as
select o.name, c.ts#, c.obj#,
       0, 1, /* i.compress$, */ nvl(c.size$, ts.blocksize), uid
from sys.obj$ o, sys.clu$ c, sys.ts$ ts /*, sys.ind$ i */
where o.obj# = c.obj#
  and ts.ts# = c.ts#
/*and c.obj# = i.bo# (+) */
  and o.type = 3
  and o.owner# = uid
/
grant select on v4expclus to public
/
remark
remark     In V5, every table and view had a row in the TABAUTH table 
remark  indicating that the table's owner had every privilege WITH
remark  GRANT OPTION.  These rows aren't created in V6, but could be
remark  added via a UNION.  Code for such a UNION is commented out below,
remark  because EXP queries this view with a WHERE clause that ensures
remark  that GRANTOR != GRANTEE.  Eliminating the UNION should improve
remark  performance.
remark     In addition, V6 has additional privileges that will be lost here,
remark  plus allows column-level grants on privileges other than UPDATE.
remark  Column-restricted grants for privileges other than UPDATE are
remark  changed to be grants for the entire table.
remark
drop view v4exptabauth
/
create view v4exptabauth (object, grantor, grantee, privs, owner) as
select o.name, ur.name, ue.name,
       decode(ta.alter$, 0, ' ', 1, 'Y', 2, 'Y', 3, 'G', ' ') ||
       decode(ta.delete$, 0, ' ', 1, 'Y', 2, 'Y', 3, 'G', ' ') ||
       decode(ta.index$, 0, ' ', 1, 'Y', 2, 'Y', 3, 'G', ' ') ||
       decode(ta.insert$, 0, ' ', 1, 'Y', 2, 'Y', 3, 'G', ' ') ||
       decode(ta.select$, 0, ' ', 1, 'Y', 2, 'Y', 3, 'G', ' ') ||
       decode(ta.update$, 0, ' ', 1, '*', 2, 'Y', 3, 'G', ' '),
       user
from sys.obj$ o, sys.user$ ur, sys.user$ ue, sys.tabauth$ ta
where o.obj# = ta.obj#
  and ur.user# = ta.grantor#
  and ue.user# = ta.grantee#
  and o.owner# = uid
  and o.type in (2, 4)
/
remark
remark  union
remark  select o.name, user, user,
remark       'GGGGGG', user
remark  from sys.obj$ o
remark  where o.owner# = uid
remark    and o.type in (2, 4)
remark
grant select on v4exptabauth to public
/
drop view v4expextents
/
create view v4expextents (pid, rba, type, sblock, eblock)
as
select t.ts#, t.obj#, 'DATA',
       e.block#, e.block# + e.length - 1
from sys.uet$ e, sys.tab$ t, sys.obj$ o
where t.file# = e.segfile#
  and t.block# = e.segblock#
  and t.obj# = o.obj#
  and o.owner# = uid
union
select cl.ts#, cl.obj#, 'DATA',
       e.block#, e.block# + e.length - 1
from sys.uet$ e, sys.clu$ cl, sys.obj$ o
where cl.file# = e.segfile#
  and cl.block# = e.segblock#
  and cl.obj# = o.obj# 
  and o.owner# = uid
union
select i.ts#, i.bo#, 'INDEX',
       e.block#, e.block# + e.length - 1
from sys.uet$ e, sys.ind$ i, sys.obj$ o
where i.file# = e.segfile#
  and i.block# = e.segblock#
  and i.bo# =  o.obj#
  and o.owner# = uid
union
select t.ts#, t.obj#, 'INDEX',
       1, decode(ts.dflinit, 1, 3, 2, 3, ts.dflinit)
from sys.tab$ t, sys.ts$ ts, sys.obj$ o
where t.ts# = ts.ts#
  and t.obj# = o.obj# 
  and o.owner# = uid
  and t.obj# not in
        (select i.bo#
         from sys.ind$ i)
union
select c.ts#, c.obj#, 'INDEX',
       1, decode(ts.dflinit, 1, 3, 2, 3, ts.dflinit)
from sys.clu$ c, sys.ts$ ts, sys.obj$ o
where c.ts# = ts.ts#
  and c.obj# = o.obj#
  and o.owner# = uid
  and c.obj# not in
        (select i.bo#
         from sys.ind$ i)
/
grant select on v4expextents to public
/
remark
remark     Views had pid's and rba's in V5, since they actually required
remark  storage in the database.  In V6, views require no storage, and
remark  thus have no file#, block#, or tab# associated with them.  
remark     V6 has a new ROWID type (69) because it is incompatible with the
remark  V5 ROWID.  This is cast to the V5 ROWID, but it may not work
remark  correctly.  (If this doesn't work, considering it a CHAR may
remark  permit a valid conversion.)  Similarly, the V6 VARCHAR datatype
remark  is converted to CHAR.  Since the V5 COL$DATATYPE column was
remark  defined to be one character too narrow to hold the words "LONG 
remark  RAW", LONG RAWs were always considered LONG in the V5 data
remark  dictionary.  Here, LONG RAW remains LONG RAW in the hopes that
remark  EXP and IMP can treat them appropriately, but this may fail.
remark  In V5, a column's default value was always NULL, and there was
remark  no way to get it into the data dictionary.  The default can be
remark  specified and stored in V6.0 (although it is ignored).  This view
remark  always return NULL for the default, however, because in V6 it is
remark  stored as a LONG and in V5 it is CHAR(240) and is useless anyway.
remark
drop view v4expcol
/
create view v4expcol
    (rba, tbl, pid, name, length, scale, type, mand, id, cmnt, dflt ) as
select t.obj#, 0, t.ts#, c.name,
       decode(c.type#, 2, nvl(c.precision, 0), c.length),
       nvl(c.scale, 0),
       decode(c.type#, 1, 'CHAR', 2, 'NUMBER', 8, 'LONG', 9, 'CHAR',
                       11, 'ROWID', 12, 'DATE', 23, 'RAW', 24, 'LONG RAW',
                       69, 'ROWID'),
       decode(c.null$, 0, 0, 1, 1, 1), c.segcol#, cc.comment$, to_char(null)
from sys.obj$ o, sys.tab$ t, sys.col$ c, sys.com$ cc
where o.obj# = t.obj#
  and t.obj# = c.obj#
  and c.obj# = cc.obj#(+)
  and c.col# = cc.col#(+)
  and o.owner# = uid
  and o.type = 2
union
select cl.obj#, 0, cl.ts#, c.name,
       decode(c.type#, 2, nvl(c.precision, 0), c.length),
       nvl(c.scale, 0),
       decode(c.type#, 1, 'CHAR', 2, 'NUMBER', 8, 'LONG', 9, 'CHAR',
                       11, 'ROWID', 12, 'DATE', 23, 'RAW', 24, 'LONG RAW',
                       69, 'ROWID'),
       decode(c.null$, 0, 0, 1), c.segcol#, cc.comment$, to_char(null)
from sys.obj$ o, sys.clu$ cl, sys.col$ c, sys.com$ cc
where o.obj# = cl.obj#
  and cl.obj# = c.obj#
  and c.obj# = cc.obj#(+)
  and c.col# = cc.col#(+)
  and o.owner# = uid
  and o.type = 3
union
select o.obj#, 0, 0, c.name,
       decode(c.type#, 2, nvl(c.precision, 0), c.length),
       nvl(c.scale, 0),
       decode(c.type#, 1, 'CHAR', 2, 'NUMBER', 8, 'LONG', 9, 'CHAR',
                       11, 'ROWID', 12, 'DATE', 23, 'RAW', 24, 'LONG RAW',
                       69, 'ROWID'),
       decode(c.null$, 0, 0, 1), c.segcol#, cc.comment$, to_char(null)
from sys.obj$ o, sys.col$ c, sys.com$ cc
where o.obj# = c.obj#
  and c.obj# = cc.obj#(+)
  and c.col# = cc.col#(+)
  and o.owner# = uid
  and o.type = 4
/
grant select on v4expcol to public
/
remark
remark  Remote synonyms are excluded here because the inconsistent way
remark  they were stored in the V5 data dictionary prevented them from
remark  appearing in the V5 V4EXPSYN view.  They wouldn't have been 
remark  exported correctly anyway because EXP doesn't look for the name
remark  of the database link.
remark
drop view v4expsyn
/
create view v4expsyn (name, sname, sowner, stime) as
select o.name, s.name, s.owner, o.ctime
from sys.obj$ o, sys.syn$ s
where o.obj# = s.obj#
  and o.type = 5
  and o.owner# = uid
  and s.node is null
/
grant select on v4expsyn to public
/
drop view v4expview
/
create view v4expview
    (name, pid, rba, tbl, aud, cmnt, vtime, vlen, vtext) as
select o.name, 0, o.obj#, 0, substr(v.audit$,1,22), c.comment$,
       o.ctime, v.textlength, v.text
from sys.obj$ o, sys.view$ v, sys.com$ c
where o.obj# = v.obj#
  and o.type = 4
  and o.obj# = c.obj#(+)
  and c.col# is null
  and o.owner# = uid
/
grant select on v4expview to public
/
